local *

{
  :ChatFrame_AddMessageEventFilter
  :GetCoinTextureString
  :InterfaceOptionsFrame_OpenToCategory
  :PVEFrame_ShowFrame
  :ReloadUI
  :SlashCmdList
  :UnitGroupRolesAssigned
  :idTweaks
  :idTweaksModule
  :idTweaksOptions
} = _G
{
  :Option
  :ToggleOption
} = idTweaksOptions

class Chat extends idTweaksModule
  new: (addon) =>
    -- print "Chat#new"

    super addon, "Chat", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0"

  set_options: =>
    -- print "Chat#set_options"

    super!

    @options\add_option Option @, "slash_commands", "Slash Commands", "Customize the Slash Commands", {},
      addon: ToggleOption @, "addon", "/idtweaks (/idt)", "Open the idTweaks Options with /idtweaks and /idt",
      reload: ToggleOption @, "reload", "/reload (/rl)", "Reload the UI with /reload and /rl"
      lfg: ToggleOption @, "lfg", "/lfg", "Open the Group Finder with /lfg"

    @options\add_option Option @, "message_style", "Message Style", "Customize the chat message styles", {},
      enabled: ToggleOption @, "enabled", "Enabled", "Change the style of your chat messages to a minimal format"

  check: =>
    -- print "Chat#check"

    @slash_commands_check!
    @message_style_check!

  enable: =>
    -- print "Chat#enable"

    @check!

  disable: =>
    -- print "Chat#disable"

    @slash_commands_disable!
    @message_style_disable!

  slash_commands_check: =>
    -- print "Chat#slash_commands_check"

    if @db.slash_commands.addon
      -- print "Chat#slash_commands_check:enabling-addon"
      SlashCmdList["IDTWEAKS_ADDON"] = (msg) ->
        @ace_module\Print "Opening options"
        InterfaceOptionsFrame_OpenToCategory "idTweaks"
        InterfaceOptionsFrame_OpenToCategory "idTweaks"
        InterfaceOptionsFrame_OpenToCategory "Automation"
      _G.SLASH_IDTWEAKS_ADDON1 = "/idt"
      _G.SLASH_IDTWEAKS_ADDON2 = "/idtweaks"
    else
      -- print "Chat#slash_commands_check:disabling-addon"
      _G.SlashCmdList["IDTWEAKS_ADDON"] = nil
      _G.SLASH_IDTWEAKS_ADDON1 = nil
      _G.SLASH_IDTWEAKS_ADDON2 = nil

    if @db.slash_commands.reload
      -- print "Chat#slash_commands_check:enabling-reload"
      _G.SlashCmdList["IDTWEAKS_RELOAD"] = ReloadUI
      _G.SLASH_IDTWEAKS_RELOAD1 = "/rl"
      _G.SLASH_IDTWEAKS_RELOAD2 = "/reload"
    else
      -- print "Chat#slash_commands_check:disabling-reload"
      _G.SlashCmdList["IDTWEAKS_RELOAD"] = nil
      _G.SLASH_IDTWEAKS_RELOAD1 = nil
      _G.SLASH_IDTWEAKS_RELOAD2 = nil

    if @db.slash_commands.lfg
      -- print "Chat#slash_commands_check:enabling-lfg"
      _G.SlashCmdList["IDTWEAKS_LFG"] = ->
        PVEFrame_ShowFrame "GroupFinderFrame"
      _G.SLASH_IDTWEAKS_LFG1 = "/lfg"
    else
      -- print "Chat#slash_commands_check:disabling-lfg"
      _G.SlashCmdList["IDTWEAKS_LFG"] = nil
      _G.SLASH_IDTWEAKS_LFG1 = nil

  message_style:
    formatters:
      money: (message) ->
        gold = message\match "(%d+) Gold"
        silver = message\match "(%d+) Silver"
        copper = message\match "(%d+) Copper"

        gold = string.format "%02d", gold if gold
        silver = string.format "%02d", silver if silver
        copper = string.format "%02d", copper if copper

        total = ""
        total = total .. gold if gold
        total = total .. silver if silver
        total = total .. copper if copper

        "+ #{GetCoinTextureString total}"
      
      role_icon: (message, sender) ->
        role = UnitGroupRolesAssigned sender

        return message unless role and role ~= "NONE"

        prefix = "\124TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES.blp:0:0:"
        suffix = "\124t"

        icon = switch role
          when "DAMAGER"
            "#{prefix}0:0:64:64:20:39:22:41#{suffix}"
          when "HEALER"
            "#{prefix}0:0:64:64:20:39:22:41#{suffix}"
          when "TANK"
            "#{prefix}0:0:64:64:0:19:22:41#{suffix}"
          when "LEADER"
            "#{prefix}0:0:64:64:0:19:1:20#{suffix}"
          when "SPACE"
            "#{prefix}0:0:64:64:0:0:0:0#{suffix}"
          else
            ""
        
        "#{icon} #{message}"

    filters:
      CHAT_MSG_BATTLEGROUND: (message, sender) ->
        Chat.message_style.formatters.role_icon message, sender
      CHAT_MSG_BATTLEGROUND_LEADER: (message, sender) ->
        Chat.message_style.formatters.role_icon message, sender
      CHAT_MSG_COMBAT_FACTION_CHANGE: (message, sender) ->
        message\gsub "Reputation with (.+) increased by (.+).", "+ %2 rep (%1)"
      CHAT_MSG_CURRENCY: (message, sender) ->
        message\gsub "You receive currency: (.+)", Chat.message_style.formatters.money message, sender
      CHAT_MSG_GUILD_ACHIEVEMENT: (message, sender) ->
        message\gsub "%%s has earned the achievement (.+)!", "\124cff00ff00g %1 %%s\124h"
      CHAT_MSG_INSTANCE_CHAT: (message, sender) ->
        Chat.message_style.formatters.role_icon message, sender
      CHAT_MSG_INSTANCE_CHAT_LEADER: (message, sender) ->
        Chat.message_style.formatters.role_icon message, sender
      CHAT_MSG_LOOT: (message, sender) ->
        result = message
        result = result\gsub "You receive item: (.+)", "+ %1"
        result = result\gsub "You receive loot: (.+)", "+ %1"
        result = result\gsub "Received item: (.+)", "+ %1"
        result
      CHAT_MSG_MONEY: (message, sender) ->
        result = message
        result = result\gsub "Received (.+) Gold, (.+) Silver, (.+) Copper", Chat.message_style.formatters.money message
        result = result\gsub "Received (.+) Silver, (.+) Copper", Chat.message_style.formatters.money message
        result = result\gsub "Received (.+) Silver", Chat.message_style.formatters.money message
        result = result\gsub "Received (.+) Copper", Chat.message_style.formatters.money message
        result = result\gsub "You loot (.+) Gold, (.+) Silver, (.+) Copper", Chat.message_style.formatters.money message
        result = result\gsub "You loot (.+) Silver, (.+) Copper", Chat.message_style.formatters.money message
        result = result\gsub "You loot (.+) Silver", Chat.message_style.formatters.money message
        result = result\gsub "You loot (.+) Copper", Chat.message_style.formatters.money message
        result
      CHAT_MSG_PARTY: (message, sender) ->
        Chat.message_style.formatters.role_icon message, sender
      CHAT_MSG_PARTY_LEADER: (message, sender) ->
        Chat.message_style.formatters.role_icon message, sender
      CHAT_MSG_RAID: (message, sender) ->
        Chat.message_style.formatters.role_icon message, sender
      CHAT_MSG_RAID_LEADER: (message, sender) ->
        Chat.message_style.formatters.role_icon message, sender
      CHAT_MSG_RAID_WARNING: (message, sender) ->
        Chat.message_style.formatters.role_icon message, sender
      CHAT_MSG_SAY: (message, sender) ->
        Chat.message_style.formatters.role_icon message, sender
      CHAT_MSG_SKILL: (message, sender) ->
        result = message
        result = result\gsub "Your skill in (.+) has increased to (.+).", "+ %2 (%1)"
        result
      CHAT_MSG_SYSTEM: (message, sender) ->
        result = message
        result = result\gsub "Received (.+) Silver", "+ %1 s"
        result = result\gsub "Received (.+) of item, (.+).", "+ %2x%1"
        result = result\gsub "(.+) completed.", "- quest (%1)"
        result = result\gsub "Quest accepted, (.+)", "+ quest (%1)"
        result = result\gsub "Received item, (.+)%.", "+ %1"
        result = result\gsub "Experience gained, (.+).", "+ %1 xp"
        result = result\gsub "(.+) has come online.", "+ %1"
        result = result\gsub "(.+) has gone offline.", "- %1"
        result = result\gsub "You are now Busy, in combat", "+ Combat"
        result = result\gsub "You are no longer marked Busy.", "- Combat"
        result = result\gsub "Discovered (.+), (.+) experience gained", "+ %2 xp (%1)"
        result = result\gsub "Quest Accepted (.+)", "+ quest (%1)"
        result = result\gsub "You are now Away, AFK", "+ AFK"
        result = result\gsub "You are no longer Away.", "- AFK"
        result = result\gsub "You are no longer rested.", "- Rested"
        result = result\gsub "You don\"t meet the requirements for that quest.", "! quest (Requirements not met)"
        result
      CHAT_MSG_WHISPER: (message, sender) ->
        Chat.message_style.formatters.role_icon message, sender
      CHAT_MSG_WHISPER_INFORM: (message, sender) ->
        Chat.message_style.formatters.role_icon message, sender
      CHAT_MSG_YELL: (message, sender) ->
        Chat.message_style.formatters.role_icon message, sender

  message_style_check: =>
    -- print "Chat#message_style_check"

    if @db.message_style.enabled
      @message_style_enable!
    else
      @message_style_disable!

  message_style_enable: =>
    -- print "Chat#message_style_enable"

    for event, filter in pairs @message_style.filters
      ChatFrame_AddMessageEventFilter event, (frame, ev, message, sender, ...) ->
        return nil, (filter message, sender), sender, ...

  -- TODO: Implement.
  message_style_disable: =>
    -- print "Chat#message_style_disable"

idTweaks\add_module Chat idTweaks
